package cerebral;

import giny.model.GraphPerspective;
import giny.model.GraphPerspectiveChangeEvent;
import giny.model.GraphPerspectiveChangeListener;
import giny.view.EdgeView;
import giny.view.NodeView;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.geom.Rectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JSplitPane;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import prefuse.Constants;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.action.ActionList;
import prefuse.action.RepaintAction;
import prefuse.action.animate.LocationAnimator;
import prefuse.activity.Activity;
import prefuse.data.Edge;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.expression.AndPredicate;
import prefuse.data.expression.BooleanLiteral;
import prefuse.data.expression.ColumnExpression;
import prefuse.data.expression.ComparisonPredicate;
import prefuse.data.expression.NumericLiteral;
import prefuse.data.expression.Predicate;
import prefuse.data.tuple.DefaultTupleSet;
import prefuse.data.tuple.TupleSet;
import prefuse.render.DefaultRendererFactory;
import prefuse.render.LabelRenderer;
import prefuse.render.PolygonRenderer;
import prefuse.render.Renderer;
import prefuse.util.ColorLib;
import prefuse.util.FontLib;
import prefuse.util.GraphicsLib;
import prefuse.util.PrefuseLib;
import prefuse.util.display.PaintListener;
import prefuse.visual.AggregateItem;
import prefuse.visual.AggregateTable;
import prefuse.visual.DecoratorItem;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualGraph;
import prefuse.visual.VisualItem;
import prefuse.visual.expression.InGroupPredicate;
import cerebral.colors.ColorManager;
import cerebral.controls.BoundedPanControl;
import cerebral.controls.BoundedZoomControl;
import cerebral.controls.ClickToSelect;
import cerebral.controls.DividerMover;
import cerebral.controls.EdgeHighlightControl;
import cerebral.controls.EdgeToolTip;
import cerebral.controls.GridDragger;
import cerebral.controls.NeighbourHighlightToGroupControl;
import cerebral.controls.PopupMenuControl;
import cerebral.controls.RubberBandSelect;
import cerebral.controls.SplineBetaController;
import cerebral.controls.ZoomViewNeighbours;
import cerebral.layout.AggregateLayout;
import cerebral.layout.LabelLayout;
import cerebral.layout.SearchGridLayout;
import cerebral.parallelCoordinates.PCPanel;
import cerebral.render.BundleEdgeRenderer;
import cerebral.render.EasyHoverShapeRenderer;
import cerebral.render.SmallLineDividerRenderer;
import cerebral.sort.LabelSorter;
import cerebral.sort.MyRenderSorter;
import cytoscape.CyNetwork;
import cytoscape.Cytoscape;
import cytoscape.data.CyAttributes;
import cytoscape.data.CyAttributesUtils;
import cytoscape.data.SelectEvent;
import cytoscape.data.SelectEventListener;
import cytoscape.view.CyNetworkView;
import cytoscape.view.CytoscapeDesktop;
import cytoscape.visual.LineType;

public class Cerebral extends Display implements SelectEventListener,
        GraphPerspectiveChangeListener, PaintListener {

    public static final String TARGET_STYLE = "targetStyle";

    public static final String SOURCE_STYLE = "sourceStyle";

    private static final String GROUP_LABELS = "groupLabels";

    public static final String GROUP_SIZE = "groupSize";

    static final String RUBBER_BAND = "rubberBand";

    public static final String SPLINE_CONTROL = "splineControl";

    public static final String LABELS = "labels";

    public static final String SINGLE_HIGHLIGHT = "singleHighlight";

    public static final String IGNORE = "ignore";

    private static final long serialVersionUID = -7249291917274938706L;

    public static final String BASE_FONT = "baseFont";

    public static final String INNATEDBREF = "InnateDBRef";

    public SearchGridLayout searchGridLayout;
    private boolean firstDisplay = true;

    private static final Schema AGGR_SCHEMA = PrefuseLib.getVisualItemSchema();
    static {
        AGGR_SCHEMA.setDefault(VisualItem.INTERACTIVE, true);
        AGGR_SCHEMA.setDefault(VisualItem.VISIBLE, false);
    }

    private static final Schema LABEL_SCHEMA = PrefuseLib.getVisualItemSchema();
    static {
        LABEL_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
    }

    private static final Schema BORDER_SCHEMA = PrefuseLib
            .getVisualItemSchema();
    static {
        BORDER_SCHEMA.setDefault(VisualItem.INTERACTIVE, false);
        BORDER_SCHEMA.setDefault(VisualItem.FILLCOLOR, ColorLib.color(ColorLib
                .getColor(0, 0, 0, 100)));
        BORDER_SCHEMA.setDefault(VisualItem.VISIBLE, false);
    }

    public static final String FUNCTION_NEIGHBOURS = "functionNeighbs";

    static final String GRAPH = "graph";

    public static final String EDGES = "graph.edges";

    public static final String NODES = "graph.nodes";

    public static final String CYTOSCAPE_ID = "cytoscapeID";

    public static final String LABEL = "label";

    public static final String STRING_ID = "stringID";

    public static final String BIO_CATEGORY = "bioCategory";

    public static final String BIO_FUNCTION = "bioFunction";

    public static final String AGGR = "aggregates";

    public static final String EDGE_SPLINE = "edgeSpline";

    public static final String EDGE_LENGTH = "edgeLength";

    public static final String SPLINE_BETA_ADJUSTED = "splineBetaAdjusted";

    public static final String SPLINE_OWNER = "splineOwner";

    public static final String EDGE_CLUSTERS = "edgeClusters";

    public static final String SELECTED = "selected";

    public static final String TEXT_COLOR = "textColor";

    public static final String FILL_COLOR = "fillColor";

    public static final String BORDER_COLOR = "borderColor";

    public static final String ANY_LAYER = "anyLayer";

    public static final String DOWNSTREAM_GENE = "downstreamGene";

    public static final String BIO_FUNCTION_UNKNOWN = "bioFunctionUnknown";

    public static final String LINE_DIVIDER = "lineDivider";

    public static final String LINE_TYPE = "lineType";

    private static final String CYTOSCAPE_DELETED = "cytoscapeDeleted";

    public static final String BUNDLE_MAX_SIZE = "bundleMaxSize";

    public static final String CLUSTER = "cluster";

    public static final String NEIGHB_SELECT = "neighbSelect";

    public static final String REORIENT = "reorient";

    public static final String PINNED = "pinned";

    public static final String LINE_DIVIDER_LABELS = "lineDividerLabels";

    public static final String FRAMING_BORDER = "framingBorder";
    
    public static final String SELF_EDGE = "selfEdge";

    private Graph graph;

    private Map cytoscapeNodeKeyToPrefuseNode;

    private Map cytoscapeEdgeKeyToPrefuseEdge;

    private Map cytoscapeNodeIDToPrefuseNode;

    private Map cytoscapeEdgeIDToPrefuseEdge;

    private Visualization vis;

    public ColorManager colorManager;

    public GroupsManager groupsManager;

    public LabelLayout labelLayout;

    public EdgeBundler edgeBundler;

    private CyNetwork cyNetwork;

    private CyNetworkView cnv;

    private ZoomViewNeighbours zoomViewNeighbours;

    private CytoscapeVisualMapper visMapper;

    private JInternalFrame cytoscapeFrame;

    private DividerMover dividerMover;

    private Table nodeTable;

    private Table edgeTable;

    private AggregateTable at;

    private Table lineDividerTable;

    private Table lineDividerLabelTable;

    private JLabel mainViewLabel;

    public ViewCoordinator viewCoordinator;

    public JInternalFrame iframe;

    public SmallMultipleContainer smallMultipleContainer;

    private double lastFontSize = 0.0;

    private double defaultFontSize = 14.0;

    private VisualGraph vg;

    /**
     * This constructor creates an action and adds it to the Plugins menu.
     * 
     * @param loadCurrentPositions
     */
    public Cerebral(CreateViewDialog layoutParameters,
            String networkIdentifier, boolean loadCurrentPositions) {

        cyNetwork = Cytoscape.getNetwork(networkIdentifier);
        NetworksManager.instance().addCerebralView(networkIdentifier, this);

        createPrefuseView(loadCurrentPositions);
        

    }

    public int getHeight() {
        return super.getHeight();
    }

    private void createPrefuseView(boolean loadCurrentPositions) {

        createCytoscapeToolbarActions();

        iframe = new JInternalFrame(cyNetwork.getTitle(), true, true, true,
                true);
        final JDesktopPane cytoDesk = Cytoscape.getDesktop()
                .getNetworkViewManager().getDesktopPane();
        Cytoscape.createNetworkView(cyNetwork);
        JInternalFrame[] allViews = cytoDesk.getAllFrames();
        for (int i = 0; i < allViews.length; i++) {
            if (allViews[i].getTitle() == cyNetwork.getTitle()) {
                cytoscapeFrame = allViews[i];
                cytoscapeFrame
                        .addInternalFrameListener(new InternalFrameAdapter() {

                            public void internalFrameClosed(
                                    InternalFrameEvent arg0) {
                                // Closing? Close our window too
                                iframe.dispose();
                            }

                            public void internalFrameActivated(
                                    InternalFrameEvent e) {
                                // Activated? Hide it and show our window
                                // instead
                                cytoscapeFrame.hide();
                                // ToDo: Link this to a real network overview.
                                // how: Create second display of visualization?
                                // Render in setNavigator window
                                // Listen for networkactivated events
                                // Update focus rectangle
                                Cytoscape.getDesktop().getNetworkPanel()
                                        .setNavigator(null);

                                try {
                                    iframe.setIcon(false);
                                    iframe.show();
                                    iframe.setSelected(true);
                                } catch (PropertyVetoException e1) {
                                    // TODO Auto-generated catch block
                                    e1.printStackTrace();
                                }
                            }
                        });
                break;
            }
        }
        cytoscapeFrame.hide();
        cytoDesk.add(iframe);

        ControlPanel.instance().addDisplay(this, cyNetwork.getIdentifier());
        iframe.addInternalFrameListener(new InternalFrameAdapter() {
            public void internalFrameActivated(InternalFrameEvent e) {

                Cytoscape.getDesktop().getSwingPropertyChangeSupport()
                        .firePropertyChange(
                                new PropertyChangeEvent(Cytoscape.getDesktop(),
                                        CytoscapeDesktop.NETWORK_VIEW_FOCUSED,
                                        null, cyNetwork.getIdentifier()));
            }

            public void internalFrameClosed(InternalFrameEvent e) {

                vis.cancel("layout");
                JInternalFrame[] allViews = cytoDesk.getAllFrames();
                for (int i = 0; i < allViews.length; i++) {
                    if (allViews[i].getTitle() == cyNetwork.getTitle()) {
                        CyNetworkView networkView = (CyNetworkView) Cytoscape
                                .getNetworkViewMap().get(
                                        cyNetwork.getIdentifier());
                        if (networkView != null) {
                            Cytoscape.destroyNetworkView(cyNetwork);
                        }
                    }
                }
                ControlPanel.instance()
                        .removeDisplay(cyNetwork.getIdentifier());
                NetworksManager.instance().removeCerebralView(
                        cyNetwork.getIdentifier());
            }

            public void internalFrameIconified(InternalFrameEvent e) {
                vis.cancel("layout");
            }

        });
        vis = new Visualization();
        setVisualization(vis);
        viewCoordinator = new ViewCoordinator(this, PCPanel.instance().getPC(
                cyNetwork.getIdentifier()));

        addPaintListener(this);
      
        loadNetwork(loadCurrentPositions);
        Dimension deskSize = cytoDesk.getSize();
        iframe.setSize(deskSize);
        iframe.setVisible(true);

        setLayout(null);
        mainViewLabel = new JLabel("");
        mainViewLabel.setBounds(10, 0, 1000, 20);
        mainViewLabel.setFont(FontLib.getFont("Verdana", 14));
        add(mainViewLabel);

        createExpressionViews();

        viewCoordinator.setQuality(ControlPanel.instance().getQuality());

        setSize(deskSize);
    }

    public void createExpressionViews() {

        List expressionAttributes = NetworksManager.instance()
                .getExpressionAttributes(cyNetwork.getIdentifier());

        iframe.getContentPane().removeAll();
        viewCoordinator.removeSmallMultiples();
        if (smallMultipleContainer == null) {
            smallMultipleContainer = new SmallMultipleContainer(this,
                    viewCoordinator);
        } else {
            smallMultipleContainer.reset();
        }
        boolean isRatioValue = true;
        if (expressionAttributes != null && expressionAttributes.size() > 0) {
            for (int i = 0; i < expressionAttributes.size(); i++) {

                graph = new Graph(nodeTable, edgeTable, false, null,
                        Graph.DEFAULT_SOURCE_KEY, Graph.DEFAULT_TARGET_KEY);

                String condition = (String) expressionAttributes.get(i);
                final SmallMultipleView view = new SmallMultipleView(condition,
                        viewCoordinator, this, cyNetwork, graph,
                        lineDividerTable, edgeBundler, edgeTable);
                smallMultipleContainer.addView(view);
                
                if (isRatioValue){
                	Map expressionValues = CyAttributesUtils.getAttribute(condition, Cytoscape
                        .getNodeAttributes());
                	Iterator it = expressionValues.values().iterator();
                	while (it.hasNext()){
                		Object attribVal = it.next();;
                		double val = 0.0;
                        if (attribVal instanceof Double) {
                        	val =((Double) attribVal).doubleValue();
                           } else if (attribVal instanceof String) {
                            try {
                            	val = Double.parseDouble((String) attribVal);
                            } catch (NumberFormatException e) {
                            }
                        } 
                        if (val < 1.0 && val > -1.0){
                        	isRatioValue = false;
                        	break;
                        }
                	}
                }
            }
            ControlPanel.instance().setIsRatioData(isRatioValue);
            viewCoordinator.enableSmallLineDivider(ControlPanel.instance()
                    .getShowSeparators());
            viewCoordinator.updateNodePositions(vis);

            JSplitPane splitPane= new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, smallMultipleContainer, this);
            splitPane.setDividerLocation((int) (iframe.getWidth()*0.3f));
            
            iframe.getContentPane().add(splitPane);

        } else {
            iframe.getContentPane().add(this);
        }

        iframe.pack();
        iframe.validate();
        try {
            iframe.setMaximum(true);
        } catch (PropertyVetoException e) {
            e.printStackTrace();
        }
    
        deselectAll();
        selectAll();

        viewCoordinator.fitViewToBounds(1000);
        viewCoordinator.updateDividers();
        vis.run("applyVizMapper");     

    }

    public void setMainLabel(String newLabel) {
        mainViewLabel.setText(newLabel);
    }

    private void saveLayoutParameters(CreateViewDialog layoutParameters) {
        CyAttributes globalAttributes = Cytoscape.getNetworkAttributes();
        globalAttributes.setAttribute(cyNetwork.getIdentifier(),
                "cerebralParameters", layoutParameters.getSaveString());

    }

    public void loadNetwork(boolean loadCurrentPositions) {
        cyNetwork.addSelectEventListener(this);
        GraphPerspective network = (GraphPerspective) cyNetwork;
        network.addGraphPerspectiveChangeListener(this);

        cytoscapeNodeKeyToPrefuseNode = new HashMap();
        cytoscapeNodeIDToPrefuseNode = new HashMap();
        nodeTable = new Table();
        nodeTable.addColumn(LABEL, String.class);
        nodeTable.addColumn(CYTOSCAPE_ID, int.class);
        nodeTable.addColumn(ANY_LAYER, boolean.class);
        nodeTable.addColumn(BIO_CATEGORY, int.class);
        nodeTable.addColumn(BIO_FUNCTION, int.class);
        nodeTable.addColumn(BIO_FUNCTION_UNKNOWN, boolean.class);
        nodeTable.addColumn(FUNCTION_NEIGHBOURS, Vector.class);
        nodeTable.addColumn(EDGE_CLUSTERS, List.class, new ArrayList());
        nodeTable.addColumn(BASE_FONT, Font.class);
        nodeTable.addColumn(TEXT_COLOR, Color.class);
        nodeTable.addColumn(FILL_COLOR, Color.class);
        nodeTable.addColumn(BORDER_COLOR, Color.class);
        nodeTable.addColumn(DOWNSTREAM_GENE, boolean.class);
        nodeTable.addColumn(CYTOSCAPE_DELETED, boolean.class);
        nodeTable.addColumn(IGNORE, boolean.class);
        nodeTable.addColumn(PINNED, boolean.class);
        nodeTable.addColumn(INNATEDBREF, String.class);
        nodeTable.addColumn(STRING_ID, String.class);

        Map innateDBRefMap = CyAttributesUtils.getAttribute(
                "InnateDB reference", Cytoscape.getNodeAttributes());

        Iterator nodesIt = network.nodesIterator();

        while (nodesIt.hasNext()) {
            giny.model.Node node = (giny.model.Node) nodesIt.next();

            int nodeId = nodeTable.addRow();
            cytoscapeNodeKeyToPrefuseNode.put((Object) node.getIdentifier(),
                    new Integer(nodeId));
            cytoscapeNodeIDToPrefuseNode.put(new Integer(node
                    .getRootGraphIndex()), new Integer(nodeId));

            nodeTable.setInt(nodeId, CYTOSCAPE_ID, node.getRootGraphIndex());
            nodeTable.setString(nodeId, STRING_ID, node.getIdentifier());
            if (innateDBRefMap != null) {
                Object att = innateDBRefMap.get(node.getIdentifier());
                if (att instanceof String) {
                    String URL = (String) att;
                    nodeTable.setString(nodeId, INNATEDBREF, URL);
                }
            }
        }

        cytoscapeEdgeKeyToPrefuseEdge = new HashMap();
        cytoscapeEdgeIDToPrefuseEdge = new HashMap();
        edgeTable = new Table();
        edgeTable.addColumn(Graph.DEFAULT_SOURCE_KEY, int.class);
        edgeTable.addColumn(Graph.DEFAULT_TARGET_KEY, int.class);
        edgeTable.addColumn(CYTOSCAPE_ID, int.class);
        edgeTable.addColumn(SPLINE_CONTROL, List.class, new ArrayList());
        edgeTable.addColumn(SPLINE_BETA_ADJUSTED, List.class, new ArrayList());
        edgeTable.addColumn(SPLINE_OWNER, NodeItem.class);
        edgeTable.addColumn(EDGE_SPLINE, Shape.class);
        edgeTable.addColumn(EDGE_LENGTH, int.class);
        edgeTable.addColumn(FILL_COLOR, Color.class);
        edgeTable.addColumn(LINE_TYPE, LineType.class);
        edgeTable.addColumn(CYTOSCAPE_DELETED, boolean.class);
        edgeTable.addColumn(BUNDLE_MAX_SIZE, int.class);
        edgeTable.addColumn(SOURCE_STYLE, int.class);
        edgeTable.addColumn(TARGET_STYLE, int.class);
        edgeTable.addColumn(REORIENT, boolean.class);
        edgeTable.addColumn(INNATEDBREF, String.class);
        edgeTable.addColumn(SELF_EDGE, boolean.class);

        innateDBRefMap = CyAttributesUtils.getAttribute("InnateDB reference",
                Cytoscape.getEdgeAttributes());
        
        
        Iterator edgesIt = network.edgesIterator();
        while (edgesIt.hasNext()) {
            giny.model.Edge edge = (giny.model.Edge) edgesIt.next();

            Integer source = (Integer) cytoscapeNodeKeyToPrefuseNode.get(edge
                    .getSource().getIdentifier());
            Integer target = (Integer) cytoscapeNodeKeyToPrefuseNode.get(edge
                    .getTarget().getIdentifier());

            int row = edgeTable.addRow();
            cytoscapeEdgeKeyToPrefuseEdge.put(edge.getIdentifier(),
                    new Integer(row));
            cytoscapeEdgeIDToPrefuseEdge.put(new Integer(edge
                    .getRootGraphIndex()), new Integer(row));

            if (innateDBRefMap != null) {
                Object att = innateDBRefMap.get(edge.getIdentifier());
                if (att instanceof String) {
                    String URL = (String) att;
                    edgeTable.setString(row, INNATEDBREF, URL);
                }
            }
            edgeTable.setInt(row, Graph.DEFAULT_SOURCE_KEY, source.intValue());
            edgeTable.setInt(row, Graph.DEFAULT_TARGET_KEY, target.intValue());
            edgeTable.setBoolean(row, SELF_EDGE, source.intValue()==target.intValue());
            edgeTable.setInt(row, CYTOSCAPE_ID, edge.getRootGraphIndex());
        }
        graph = new Graph(nodeTable, edgeTable, false, null,
                Graph.DEFAULT_SOURCE_KEY, Graph.DEFAULT_TARGET_KEY);

        vg = vis.addGraph(GRAPH, graph);
        vis.setInteractive(EDGES, null, true);
        vis.addDecorators(LABELS, NODES, LABEL_SCHEMA);
        
        
        vis.addFocusGroup(SINGLE_HIGHLIGHT, new DefaultTupleSet());
        TupleSet selectedItems = new DefaultTupleSet();
        vis.addFocusGroup(SELECTED, selectedItems);

        TupleSet hoverItems = new DefaultTupleSet();
        vis.addFocusGroup(NEIGHB_SELECT, hoverItems);

        // Create table for storing aggregate hull locations
        at = vis.addAggregates(AGGR, AGGR_SCHEMA);
        at.addColumn(LABEL, String.class);
        at.addColumn(BIO_FUNCTION, int.class);
        at.addColumn(GROUP_SIZE, int.class);
        at.addColumn(VisualItem.POLYGON, float[].class);

        if (loadCurrentPositions) {
            // Map exisitng cytoscape positions to initial positions of prefuse
            cnv = Cytoscape.getNetworkView(cyNetwork.getIdentifier());
            if (cnv != null) {
                nodesIt = network.nodesIterator();
                while (nodesIt.hasNext()) {
                    giny.model.Node cytoNode = (giny.model.Node) nodesIt.next();
                    Integer id = (Integer) cytoscapeNodeKeyToPrefuseNode
                            .get(cytoNode.getIdentifier());
                    Node node = graph.getNode(id.intValue());
                    NodeItem nodeItem = (NodeItem) vis.getVisualItem(NODES,
                            node);

                    NodeView nodeView = cnv.getNodeView(cytoNode);
                    if (nodeView != null) {
                        nodeItem.setX(nodeView.getXPosition());
                        nodeItem.setEndX(nodeView.getXPosition());
                        nodeItem.setY(nodeView.getYPosition());
                        nodeItem.setEndY(nodeView.getYPosition());
                        if (nodeView.isSelected()) {
                            selectedItems.addTuple(nodeItem);
                        }
                    }

                    edgesIt = network.edgesIterator();
                    while (edgesIt.hasNext()) {
                        giny.model.Edge cytoEdge = (giny.model.Edge) edgesIt
                                .next();

                        EdgeView edgeView = cnv.getEdgeView(cytoEdge);
                        if (edgeView != null) {
                            if (edgeView.isSelected()) {
                                id = (Integer) cytoscapeEdgeKeyToPrefuseEdge
                                        .get(cytoEdge.getIdentifier());
                                prefuse.data.Edge edge = graph.getEdge(id
                                        .intValue());
                                EdgeItem edgeItem = (EdgeItem) vis
                                        .getVisualItem(EDGES, edge);
                                selectedItems.addTuple(edgeItem);
                            }
                        }
                    }
                }

            }
        }

        vis.addDecorators(GROUP_LABELS, AGGR, LABEL_SCHEMA);

        Table rubberBandTable = new Table();
        rubberBandTable.addColumn(VisualItem.POLYGON, float[].class);
        int r = rubberBandTable.addRow();
        rubberBandTable.set(r, VisualItem.POLYGON, new float[8]);
        vis.add(RUBBER_BAND, rubberBandTable);
        VisualItem rubberBand = (VisualItem) vis.getVisualGroup(RUBBER_BAND)
                .tuples().next();
        rubberBand.setStrokeColor(ColorLib.color(ColorLib.getColor(255, 0, 0)));

        lineDividerTable = new Table();
        vis.add(LINE_DIVIDER, lineDividerTable);
        vis.setInteractive(LINE_DIVIDER, null, false);

        lineDividerLabelTable = new Table();
        lineDividerLabelTable.addColumn(LABEL, String.class);
        vis.add(LINE_DIVIDER_LABELS, lineDividerLabelTable);
        vis.setInteractive(LINE_DIVIDER_LABELS, null, false);

        /*
        // The framing table frames the main view to show the zoom level
        // of the small multiple views
        Table framingBorderTable = new Table();
        framingBorderTable.addColumn(VisualItem.POLYGON, float[].class);
        framingBorderTable.addRows(4);
        for (int i = 0; i < 4; i++) {
            framingBorderTable.set(i, VisualItem.POLYGON, new float[8]);
        }
        vis.addTable(FRAMING_BORDER, framingBorderTable, BORDER_SCHEMA);
*/
        colorManager = new ColorManager();

        Renderer polyR = new PolygonRenderer(Constants.POLY_TYPE_CURVE);
        ((PolygonRenderer) polyR).setCurveSlack(0.15f);

        EasyHoverShapeRenderer nodeRenderer = new EasyHoverShapeRenderer(this);
        nodeRenderer.setBaseSize(20);

        BundleEdgeRenderer edgeRenderer = new BundleEdgeRenderer();

        DefaultRendererFactory drf = new DefaultRendererFactory(nodeRenderer,
                edgeRenderer);

        Renderer rubberBandRenderer = new PolygonRenderer(
                Constants.POLY_TYPE_LINE);
        SmallLineDividerRenderer lineDividerRenderer = new SmallLineDividerRenderer(
                vis);

        drf.add(new InGroupPredicate(AGGR), polyR);
        drf.add(new InGroupPredicate(LABELS), new LabelRenderer());
        drf.add(new InGroupPredicate(GROUP_LABELS), new LabelRenderer());
        LabelRenderer lineDividerLabelRenderer = new LabelRenderer();
        lineDividerLabelRenderer.setHorizontalTextAlignment(Constants.LEFT);
        lineDividerLabelRenderer.setHorizontalAlignment(Constants.LEFT);
        drf.add(new InGroupPredicate(LINE_DIVIDER_LABELS),
                lineDividerLabelRenderer);
        drf.add(new InGroupPredicate(RUBBER_BAND), rubberBandRenderer);
        drf.add(new InGroupPredicate(LINE_DIVIDER), lineDividerRenderer);
 //       drf.add(new InGroupPredicate(FRAMING_BORDER), rubberBandRenderer);
        vis.setRendererFactory(drf);

        // -- the processing actions ---------------------------------------

        labelLayout = new LabelLayout(new LabelSorter(), this);
        labelLayout.setExtraSpace(ControlPanel.instance().getDensity());
        labelLayout.setAggregateLabelScale(ControlPanel.instance()
                .getGroupLabelSize());

        edgeBundler = new EdgeBundler(vis.getVisualGroup(NODES), vis
                .getVisualGroup(EDGES));
        edgeBundler.recomputeLocations();
        edgeBundler.setBeta(ControlPanel.instance().getBeta());

        visMapper = new CytoscapeVisualMapper(vis, cyNetwork, colorManager,
                viewCoordinator, edgeBundler);
        Cytoscape.getVisualMappingManager().addChangeListener(visMapper);

        // add the actions to the visualization
        colorManager.createColorRules(vis, labelLayout, edgeTable);

        ActionList animateNodes = new ActionList(1000);
        animateNodes.add(new LocationAnimator(NODES));
        animateNodes.add(new AggregateLayout(AGGR));
        animateNodes.add(new RepaintAction());
        vis.putAction("animateNodes", animateNodes);

        setItemSorter(new MyRenderSorter(edgeTable));

        createDisplayControls(vg, labelLayout, edgeBundler, edgeTable,
                rubberBand);

        groupsManager = new GroupsManager(vis);

    }

    public void loadLayoutParameters(CreateViewDialog layoutParameters) {

        saveLayoutParameters(layoutParameters);

        Iterator nodesIt = vis.getVisualGroup(NODES).tuples();
        while (nodesIt.hasNext()) {
            NodeItem nodeItem = (NodeItem) nodesIt.next();
            giny.model.Node ginyNode = getCytoscapeNode(nodeItem);

            if (layoutParameters.placeAnyLayer(ginyNode)) {
                nodeItem.setBoolean(ANY_LAYER, true);
            } else {
                nodeItem.setInt(BIO_CATEGORY, layoutParameters
                        .getLayer(ginyNode));
            }
            if (layoutParameters.bioFunctionUnknown(ginyNode)) {
                nodeItem.setBoolean(BIO_FUNCTION_UNKNOWN, true);
            } else {
                nodeItem.setInt(BIO_FUNCTION, layoutParameters
                        .getFunction(ginyNode));
            }
        }

        assignDownstreamGenes(layoutParameters, cyNetwork, nodeTable);

        for (int j = 0; j < layoutParameters.getNumLayers(); j++) {
            for (int i = 0; i < layoutParameters.getNumFunctions(); i++) {
                Predicate functionGroup = new AndPredicate(
                        new ComparisonPredicate(ComparisonPredicate.EQ,
                                new ColumnExpression(Cerebral.BIO_FUNCTION),
                                new NumericLiteral(i)),
                        new ComparisonPredicate(ComparisonPredicate.EQ,
                                new ColumnExpression(Cerebral.BIO_CATEGORY),
                                new NumericLiteral(j)));
                Iterator functionGroupIt = vis.getVisualGroup(NODES).tuples(
                        functionGroup);
                Vector group = new Vector();
                while (functionGroupIt.hasNext()) {
                    NodeItem node = (NodeItem) functionGroupIt.next();
                    group.add(node);
                    node.set(FUNCTION_NEIGHBOURS, group);
                }
            }
        }

        at.clear();

        for (int i = 0; i < layoutParameters.getNumFunctions(); i++) {
            Predicate functionGroup = new AndPredicate(new ComparisonPredicate(
                    ComparisonPredicate.EQ, new ColumnExpression(
                            Cerebral.BIO_FUNCTION), new NumericLiteral(i)),
                    new ComparisonPredicate(ComparisonPredicate.EQ,
                            new ColumnExpression(Cerebral.DOWNSTREAM_GENE),
                            new BooleanLiteral(true)));
            Iterator functionGroupIt = vis.getVisualGroup(NODES).tuples(
                    functionGroup);
            AggregateItem ai = (AggregateItem) at.addItem();
            String label = layoutParameters.getFunctionName(i);
            ai.setString(LABEL, label.replace(" ", "\n"));
            ai.setInt(BIO_FUNCTION, i);
            Vector group = new Vector();
            while (functionGroupIt.hasNext()) {
                NodeItem node = (NodeItem) functionGroupIt.next();
                group.add(node);
                node.set(FUNCTION_NEIGHBOURS, group);
                ai.addItem(node);
            }
            ai.setInt(GROUP_SIZE, group.size());
        }

        {
            DecoratorItem labels[] = new DecoratorItem[vis.getGroup(LABELS)
                    .getTupleCount()
                    + vis.getGroup(GROUP_LABELS).getTupleCount()];

            Iterator iter = vis.items(LABELS);
            int i = 0;
            while (iter.hasNext()) {
                labels[i++] = (DecoratorItem) (iter.next());
            }

            iter = vis.items(GROUP_LABELS);
            while (iter.hasNext()) {
                labels[i++] = (DecoratorItem) (iter.next());
            }

            labelLayout.setLabelsToManage(labels);
        }

        int targetLayers = layoutParameters.getNumLayers() - 1;
        targetLayers += layoutParameters.displayDownstreamGenes() ? 1 : 0;

        while (lineDividerTable.getRowCount() < targetLayers) {
            lineDividerTable.addRow();
        }
        while (lineDividerTable.getRowCount() > targetLayers) {
            lineDividerTable.removeRow(lineDividerTable.getRowCount()-1);
        }

        lineDividerLabelTable.clear();
        for (int i = 0; i < layoutParameters.getNumLayers(); i++) {
            int id = lineDividerLabelTable.addRow();
            lineDividerLabelTable.setString(id, LABEL, layoutParameters
                    .getLayerName(i));
        }
        if (layoutParameters.displayDownstreamGenes()) {
            int id = lineDividerLabelTable.addRow();
            lineDividerLabelTable.setString(id, LABEL, layoutParameters
                    .getDownstreamLabel());
        }

        dividerMover.load();
        viewCoordinator.enableSmallLineDivider(ControlPanel.instance()
                .getShowSeparators());
        setShowHulls(ControlPanel.instance().getShowHulls());
        setHullColor(ControlPanel.instance().getHullColor());

        ActionList layout = new ActionList(Activity.INFINITY);
        searchGridLayout = new SearchGridLayout(vg, "layout", layoutParameters
                .getNumLayers());
        layout.add(searchGridLayout);
        vis.putAction("layout", layout);
        vis.putAction("repaint", new RepaintAction());
        vis.putAction("aggLayout", new AggregateLayout(AGGR));
        vis.putAction("applyVizMapper", visMapper);
        
        vis.run("aggLayout");
        vis.run("color");
        vis.run("repaint");

        addComponentListener(new ComponentAdapter(){
			public void componentResized (ComponentEvent e) {
				if (firstDisplay){
					firstDisplay=false;
					doLayout();
					viewCoordinator.fitViewToBounds(1000);
				}
			}    
			
        });
    }

    public void setLayerLabelPositions() {
        Rectangle2D r = Cerebral.getEndBounds(vis, Cerebral.NODES);
        for (int i = 0; i < lineDividerTable.getRowCount(); i++) {
            double dividerPos = vis.getVisualItem(LINE_DIVIDER,
                    lineDividerTable.getTuple(i)).getY();
           
            double labelPos =0;
            int prevDivider = i - 1;
            double prevPos = r.getMinY();
            if (prevDivider >= 0) {
                prevPos = vis.getVisualItem(LINE_DIVIDER,
                        lineDividerTable.getTuple(prevDivider)).getY();
                labelPos = (prevPos + dividerPos)/2.0;
            }else{
                labelPos = dividerPos - 30;
            }
            VisualItem dividerLabel = vis.getVisualItem(LINE_DIVIDER_LABELS,
                    lineDividerLabelTable.getTuple(i));
            dividerLabel.setY(labelPos);
            dividerLabel.setX(r.getMaxX() + SearchGridLayout.SCALE);
            dividerLabel.setTextColor(ColorLib.gray(0));
            dividerLabel.setFont(FontLib.getFont("Verdana", 14.0));
        }

        if (lineDividerLabelTable.getRowCount() > lineDividerTable
                .getRowCount()
                && lineDividerTable.getRowCount() > 0) {
            // position downstream label
            double dividerPos = vis.getVisualItem(
                    LINE_DIVIDER,
                    lineDividerTable
                            .getTuple(lineDividerTable.getRowCount() - 1))
                    .getY();
            VisualItem dividerLabel = vis.getVisualItem(LINE_DIVIDER_LABELS,
                    lineDividerLabelTable.getTuple(lineDividerLabelTable
                            .getRowCount() - 1));
            dividerLabel.setY((dividerPos + r.getMaxY()) / 2.0);
            dividerLabel.setX(r.getMaxX() + SearchGridLayout.SCALE);
            dividerLabel.setTextColor(ColorLib.gray(0));
            dividerLabel.setFont(FontLib.getFont("Verdana", 14.0));
            dividerLabel.setTextColor(ColorLib.gray(0));
            lastFontSize = -1;

        }

    }

    private void createCytoscapeToolbarActions() {
        // Try to add a listener for the toolbars
        Component[] toolbarButtons = Cytoscape.getDesktop().getCyMenus()
                .getToolBar().getComponents();
        for (int i = 0; i < toolbarButtons.length; i++) {

            if (toolbarButtons[i] instanceof JButton) {
                JButton button = (JButton) toolbarButtons[i];
                String tooltipText = button.getToolTipText();
                if (tooltipText != null) {
                    if (button.getToolTipText().compareToIgnoreCase("Zoom Out") == 0) {
                        button.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                viewCoordinator.zoom(0.9);

                            }
                        });
                    }
                    if (button.getToolTipText().compareToIgnoreCase("Zoom In") == 0) {
                        button.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                viewCoordinator.zoom(1.1);
                            }
                        });
                    }

                    if (button.getToolTipText().compareToIgnoreCase(
                            "Zoom out to display all of current Network") == 0) {
                        button.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {      
                            	viewCoordinator.fitViewToBounds(1000); 
                            }
                        });
                    }
                    if (button.getToolTipText().compareToIgnoreCase(
                            "Zoom Selected Region") == 0) {
                        button.addActionListener(new ActionListener() {
                            public void actionPerformed(ActionEvent e) {
                                TupleSet selectedItems = vis
                                        .getFocusGroup(SELECTED);
                                Iterator it = selectedItems.tuples();
                                Rectangle2D bounds = null;
                                while (it.hasNext()) {
                                    VisualItem vi = (VisualItem) it.next();
                                    if (vi instanceof NodeItem) {
                                        if (bounds == null) {
                                            Rectangle2D initBounds = vi
                                                    .getBounds();
                                            bounds = new Rectangle2D.Double(
                                                    initBounds.getX(),
                                                    initBounds.getY(),
                                                    initBounds.getWidth(),
                                                    initBounds.getHeight());
                                        } else {
                                            Rectangle2D.union(bounds, vi
                                                    .getBounds(), bounds);
                                        }
                                    }
                                }
                                if (bounds != null) {
                                    GraphicsLib.expand(bounds,
                                            50 + (int) (1 / getScale()));
                                    viewCoordinator.fitViewToBounds(bounds,
                                            2000);
                                }
                            }
                        });
                    }
                    if (button.getToolTipText().equalsIgnoreCase(
                            "Show all Nodes and Edges (unhiding as necessary)")) {
                        button.addActionListener(new ActionListener() {

                            public void actionPerformed(ActionEvent e) {
                                Iterator nodesIt = vis.getVisualGroup(NODES)
                                        .tuples();
                                while (nodesIt.hasNext()) {
                                    NodeItem nodeItem = (NodeItem) nodesIt
                                            .next();
                                    if (!nodeItem.getBoolean(CYTOSCAPE_DELETED)) {
                                        nodeItem.setVisible(true);
                                    }
                                }
                                Iterator edgesIt = vis.getVisualGroup(EDGES)
                                        .tuples();
                                while (edgesIt.hasNext()) {
                                    EdgeItem edgeItem = (EdgeItem) edgesIt
                                            .next();
                                    if (!edgeItem.getBoolean(CYTOSCAPE_DELETED)) {
                                        edgeItem.setVisible(true);
                                    }
                                }
                                showAggregatesIfAnyMemberVisible();
                                vis.run("color");
                            }
                        });
                    }
                    if (button.getToolTipText().equalsIgnoreCase(
                            "Hide Selected Region")) {
                        button.addActionListener(new ActionListener() {

                            public void actionPerformed(ActionEvent e) {
                                Iterator nodesIt = vis.getVisualGroup(NODES)
                                        .tuples();
                                List deselect = new ArrayList();
                                while (nodesIt.hasNext()) {
                                    NodeItem nodeItem = (NodeItem) nodesIt
                                            .next();
                                    if (nodeItem.isInGroup(SELECTED)) {
                                        nodeItem.setVisible(false);
                                        deselect.add(nodeItem);
                                    }
                                }
                                Iterator edgesIt = vis.getVisualGroup(EDGES)
                                        .tuples();
                                while (edgesIt.hasNext()) {
                                    EdgeItem edgeItem = (EdgeItem) edgesIt
                                            .next();
                                    if (edgeItem.isInGroup(SELECTED)) {
                                        edgeItem.setVisible(false);
                                        deselect.add(edgeItem);
                                    }
                                }
                                selectInCytoscape(deselect.toArray(), false);

                                hideAggregatesIfAllMembersHidden();

                                vis.run("color");
                            }

                        });
                    }
                }
            }
        }
    }

    private void createDisplayControls(VisualGraph vg, LabelLayout ll,
            EdgeBundler edgeBundler, Table edges, VisualItem rubberBand) {
        addControlListener(new GridDragger(vis, vg.getNodes(), this,
                edgeBundler));
        addControlListener(new BoundedPanControl(
                prefuse.controls.Control.MIDDLE_MOUSE_BUTTON, NODES,
                viewCoordinator));

        addPaintListener(ll);

        RubberBandSelect rubberBandSelect = new RubberBandSelect(this,
                rubberBand);
        addControlListener(rubberBandSelect);
        addControlListener(new BoundedZoomControl(NODES, viewCoordinator));
        addControlListener(new SplineBetaController(edgeBundler));

        addControlListener(new NeighbourHighlightToGroupControl("color",
                NEIGHB_SELECT, viewCoordinator));

        addControlListener(new EdgeHighlightControl("color", NEIGHB_SELECT,
                viewCoordinator));
        zoomViewNeighbours = new ZoomViewNeighbours(vis, vis
                .getFocusGroup(SINGLE_HIGHLIGHT));
        addControlListener(zoomViewNeighbours);

        // neighbourSelect = new
        // NeighbourSelect(vis.getFocusGroup(SINGLE_HIGHLIGHT));
        // addControlListener(neighbourSelect);
        addControlListener(new ClickToSelect(this));
        addControlListener(new PopupMenuControl(this, vis));        
        addControlListener(new EdgeToolTip(this,"interaction"));

        dividerMover = new DividerMover(viewCoordinator,vis, vis.getVisualGroup(LINE_DIVIDER));
        addControlListener(dividerMover);
    }

    public void setSelectedItems(Set visualItems) {
        TupleSet curSelected = vis.getFocusGroup(SELECTED);
        List deselectNodes = new ArrayList();
        List deselectEdges = new ArrayList();
        Iterator it = curSelected.tuples();
        while (it.hasNext()) {
            VisualItem item = (VisualItem) it.next();
            if (!visualItems.contains(item)) {
                if (item instanceof NodeItem) {
                    deselectNodes.add(getCytoscapeNode(item));
                } else if (item instanceof EdgeItem) {
                    deselectEdges.add(getCytoscapeEdge(item));
                }
            }
        }
        cyNetwork.setSelectedNodeState(deselectNodes, false);
        cyNetwork.setSelectedEdgeState(deselectEdges, false);

        it = visualItems.iterator();
        List selectNodes = new ArrayList();
        List selectEdges = new ArrayList();
        while (it.hasNext()) {
            VisualItem item = (VisualItem) it.next();
            if (item instanceof NodeItem) {
                selectNodes.add(getCytoscapeNode(item));
            } else if (item instanceof EdgeItem) {
                selectEdges.add(getCytoscapeEdge(item));
            }
        }
        cyNetwork.setSelectedNodeState(selectNodes, true);
        cyNetwork.setSelectedEdgeState(selectEdges, true);

    }

    private giny.model.Node getCytoscapeNode(VisualItem nodeItem) {
        int id = nodeItem.getInt(CYTOSCAPE_ID);
        return cyNetwork.getNode(id);
    }

    public giny.model.Edge getCytoscapeEdge(VisualItem edgeItem) {
        int id = edgeItem.getInt(CYTOSCAPE_ID);
        return cyNetwork.getEdge(id);
    }

    public void selectInCytoscape(Object[] visualItems, boolean selectState) {
        List cytoNodes = new ArrayList();
        List cytoEdges = new ArrayList();
        for (int i = 0; i < visualItems.length; i++) {
            Object obj = visualItems[i];
            if (obj instanceof VisualItem) {
                VisualItem vi = (VisualItem) obj;
                if (vi.isVisible() && !vi.getBoolean(CYTOSCAPE_DELETED)) {
                    if (vi instanceof NodeItem) {
                        cytoNodes.add(getCytoscapeNode(vi));
                    } else if (vi instanceof EdgeItem) {
                        cytoEdges.add(getCytoscapeEdge(vi));
                    }
                }
            } else {
                System.out.println("I don't know how to deal with this item.");
            }
        }
        if (cytoNodes.size() > 0 || cytoEdges.size() > 0) {
            cyNetwork.setSelectedNodeState(cytoNodes, selectState);
            cyNetwork.setSelectedEdgeState(cytoEdges, selectState);
        }
    }

    private void assignDownstreamGenes(CreateViewDialog layoutParameters,
            GraphPerspective network, Table nodeTable) {

        if (layoutParameters.downstreamIsEdgeAttribute()) {
            Iterator edgeIt = network.edgesIterator();
            while (edgeIt.hasNext()) {
                giny.model.Edge edge = (giny.model.Edge) edgeIt.next();
                giny.model.Node target = edge.getTarget();
                Integer row = (Integer) cytoscapeNodeKeyToPrefuseNode
                        .get(target.getIdentifier());
                nodeTable.setBoolean(row.intValue(), DOWNSTREAM_GENE,
                        layoutParameters.isDownstreamGene(edge));

            }
        } else {
            Iterator nodeIt = network.nodesIterator();
            while (nodeIt.hasNext()) {
                giny.model.Node node = (giny.model.Node) nodeIt.next();

                Integer row = (Integer) cytoscapeNodeKeyToPrefuseNode.get(node
                        .getIdentifier());
                nodeTable.setBoolean(row.intValue(), DOWNSTREAM_GENE,
                        layoutParameters.isDownstreamGene(node));

            }

        }
    }

    public void onSelectEvent(SelectEvent se) {

        TupleSet selected = vis.getFocusGroup(SELECTED);
        // Cytoscape generated select event
        boolean selecting = se.getEventType();
        if (se.getTargetType() == SelectEvent.NODE_SET) {
            Collection cytoNodes = (Collection) se.getTarget();

            Iterator it = cytoNodes.iterator();
            while (it.hasNext()) {
                giny.model.Node cytoNode = (giny.model.Node) it.next();
                Integer key = (Integer) cytoscapeNodeKeyToPrefuseNode
                        .get(cytoNode.getIdentifier());
                if (key != null) {
                    prefuse.data.Node node = graph.getNodeFromKey(key
                            .intValue());
                    NodeItem nodeItem = (NodeItem) vis.getVisualItem(NODES,
                            node);
                    if (selecting)
                        selected.addTuple(nodeItem);
                    else
                        selected.removeTuple(nodeItem);
                    viewCoordinator.select(SELECTED, node, selecting);                   
                }
            }
        } else if (se.getTargetType() == SelectEvent.SINGLE_NODE) {
            giny.model.Node cytoNode = (giny.model.Node) se.getTarget();
            Integer key = (Integer) cytoscapeNodeKeyToPrefuseNode.get(cytoNode
                    .getIdentifier());
            if (key != null) {
                prefuse.data.Node node = graph.getNodeFromKey(key.intValue());
                NodeItem nodeItem = (NodeItem) vis.getVisualItem(NODES, node);
                if (selecting)
                    selected.addTuple(nodeItem);
                else
                    selected.removeTuple(nodeItem);

                viewCoordinator.select(SELECTED, node, selecting);
            }
        } else if (se.getTargetType() == SelectEvent.EDGE_SET) {
            Collection cytoEdges = (Collection) se.getTarget();
            Iterator it = cytoEdges.iterator();
            while (it.hasNext()) {
                giny.model.Edge cytoEdge = (giny.model.Edge) it.next();
                Integer key = (Integer) cytoscapeEdgeKeyToPrefuseEdge
                        .get(cytoEdge.getIdentifier());
                if (key != null) {
                    prefuse.data.Edge edge = graph.getEdge(key.intValue());
                    EdgeItem edgeItem = (EdgeItem) vis.getVisualItem(EDGES,
                            edge);
                    if (selecting)
                        selected.addTuple(edgeItem);
                    else
                        selected.removeTuple(edgeItem);

                    viewCoordinator.select(SELECTED, edgeItem, selecting);
                }
            }
        } else if (se.getTargetType() == SelectEvent.SINGLE_EDGE) {
            giny.model.Edge cytoEdge = (giny.model.Edge) se.getTarget();
            Integer key = (Integer) cytoscapeEdgeKeyToPrefuseEdge.get(cytoEdge
                    .getIdentifier());
            if (key != null) {
                prefuse.data.Edge edge = graph.getEdge(key.intValue());
                EdgeItem edgeItem = (EdgeItem) vis.getVisualItem(EDGES, edge);
                if (selecting)
                    selected.addTuple(edgeItem);
                else
                    selected.removeTuple(edgeItem);

                viewCoordinator.select(SELECTED, edgeItem, selecting);
            }
        }

        viewCoordinator.run("color");
        viewCoordinator.run("animate");

    }

    public void setCytoscapePosition(NodeItem nodeItem, boolean updateCurX) {
        if (cnv == null) {
            cnv = Cytoscape.createNetworkView(cyNetwork);
        }

        int cytoIndex = nodeItem.getInt(Cerebral.CYTOSCAPE_ID);
        giny.model.Node cytoNode = cyNetwork.getNode(cytoIndex);
        NodeView nodeView = cnv.getNodeView(cytoNode);
        if (nodeView != null) {
            nodeView.setXPosition(nodeItem.getEndX());
            nodeView.setYPosition(nodeItem.getEndY());
        }

        viewCoordinator.updateNodePosition(nodeItem, updateCurX);
    }

    public void setCytoscapePostionsFromLayout(TupleSet nodeItems) {
        Iterator it = nodeItems.tuples();
        while (it.hasNext()) {
            NodeItem nodeItem = (NodeItem) it.next();
            setCytoscapePosition(nodeItem, false);
        }
        viewCoordinator.repaintAll();
    }

    public void setShowHulls(boolean show) {
        Iterator aggregateIt = vis.getVisualGroup(AGGR).tuples();
        boolean redraw = false;
        while (aggregateIt.hasNext()) {
            AggregateItem aggItem = (AggregateItem) aggregateIt.next();
            if (aggItem.isVisible() != show) {
                aggItem.setVisible(show);
                redraw = true;
            } else {
                break;
            }
        }
        if (redraw) {
            vis.run("aggLayout");
            vis.run("color");
        }
    }

    public void setHullColor(Color hullColor) {

        Iterator aggregateIt = vis.getVisualGroup(AGGR).tuples();
        while (aggregateIt.hasNext()) {
            AggregateItem aggItem = (AggregateItem) aggregateIt.next();
            aggItem.setFillColor(ColorLib.color(hullColor));
        }
        vis.run("color");
    }

    public void setShowSeparators(boolean showSeparators) {
        viewCoordinator.enableSmallLineDivider(showSeparators);
    }

    public NodeItem getNodeItemFromCytoID(String objectKey) {
        Integer id = (Integer) cytoscapeNodeKeyToPrefuseNode.get(objectKey);
        Node node = graph.getNode(id.intValue());
        NodeItem nodeItem = (NodeItem) vis.getVisualItem(NODES, node);
        return nodeItem;
    }

    public NodeItem getNodeItemFromCytoID(int id) {
        Integer prefuseId = (Integer) cytoscapeNodeIDToPrefuseNode
                .get(new Integer(id));
        if (prefuseId != null) {
            Node node = graph.getNode(prefuseId.intValue());
            NodeItem nodeItem = (NodeItem) vis.getVisualItem(NODES, node);
            return nodeItem;
        }
        return null;
    }

    public EdgeItem getEdgeItemFromCytoID(int id) {
        Integer prefuseId = (Integer) cytoscapeEdgeIDToPrefuseEdge
                .get(new Integer(id));
        if (prefuseId != null) {
            Edge edge = graph.getEdge(prefuseId.intValue());
            EdgeItem edgeItem = (EdgeItem) vis.getVisualItem(EDGES, edge);
            return edgeItem;
        }
        return null;
    }

    public void graphPerspectiveChanged(GraphPerspectiveChangeEvent e) {

        // Cytoscape lies. These events don't get generated because edges/nodes
        // are hidden
        // They get generated because these edges/nodes are removed.
        switch (e.getType()) {
        case GraphPerspectiveChangeEvent.EDGES_HIDDEN_TYPE: {
            int[] edgeIndices = e.getHiddenEdgeIndices();
            for (int i = 0; i < edgeIndices.length; i++) {
                EdgeItem edgeItem = getEdgeItemFromCytoID(edgeIndices[i]);
                if (edgeItem != null) {
                    edgeItem.setVisible(false);
                    viewCoordinator.updateVisible(edgeItem);
                    edgeItem.setBoolean(CYTOSCAPE_DELETED, true);
                }
            }
            break;
        }
        case GraphPerspectiveChangeEvent.NODES_HIDDEN_TYPE: {
            int[] nodeIndices = e.getHiddenNodeIndices();
            for (int i = 0; i < nodeIndices.length; i++) {
                NodeItem nodeItem = getNodeItemFromCytoID(nodeIndices[i]);

                if (nodeItem != null) {
                    nodeItem.setVisible(false);
                    viewCoordinator.updateVisible(nodeItem);
                    nodeItem.setBoolean(CYTOSCAPE_DELETED, true);
                    hideAggregatesIfAllMembersHidden();
                }
            }
            break;
        }
        case GraphPerspectiveChangeEvent.NODES_RESTORED_TYPE: {
            int[] nodeIndices = e.getRestoredNodeIndices();
            for (int i = 0; i < nodeIndices.length; i++) {
                NodeItem nodeItem = getNodeItemFromCytoID(nodeIndices[i]);

                if (nodeItem != null) {
                    nodeItem.setVisible(true);
                    viewCoordinator.updateVisible(nodeItem);
                    nodeItem.setBoolean(CYTOSCAPE_DELETED, false);
                    showAggregatesIfAnyMemberVisible();
                }
            }
            break;
        }
        case GraphPerspectiveChangeEvent.EDGES_RESTORED_TYPE: {
            int[] edgeIndices = e.getRestoredEdgeIndices();
            for (int i = 0; i < edgeIndices.length; i++) {
                EdgeItem edgeItem = getEdgeItemFromCytoID(edgeIndices[i]);
                if (edgeItem != null) {
                    edgeItem.setVisible(true);
                    viewCoordinator.updateVisible(edgeItem);
                    edgeItem.setBoolean(CYTOSCAPE_DELETED, false);
                }
            }
            break;

        }
        default:
            System.out
                    .println("Received an unexpected GraphPerspectiveChangeEvent");
        }
        viewCoordinator.run("aggLayout");
        viewCoordinator.run("color");
        viewCoordinator.repaintAll();

    }

    private void showAggregatesIfAnyMemberVisible() {
        Iterator aggregateIt = vis.getVisualGroup(AGGR).tuples();
        while (aggregateIt.hasNext()) {
            AggregateItem aggItem = (AggregateItem) aggregateIt.next();
            boolean atLeastOneVisible = false;
            Iterator items = aggItem.items();
            while (items.hasNext()) {
                VisualItem item = (VisualItem) items.next();
                if (item.isVisible()) {
                    atLeastOneVisible = true;
                    break;
                }
            }
            if (atLeastOneVisible) {
                aggItem.setVisible(true);
            }
        }
    }

    private void hideAggregatesIfAllMembersHidden() {
        // See if we hid everything in an aggregate group
        Iterator aggregateIt = vis.getVisualGroup(AGGR).tuples();
        while (aggregateIt.hasNext()) {
            AggregateItem aggItem = (AggregateItem) aggregateIt.next();
            Iterator membersIt = aggItem.items();
            boolean allInvisible = true;
            while (membersIt.hasNext()) {
                NodeItem nodeItem = (NodeItem) membersIt.next();
                if (nodeItem.isVisible()) {
                    allInvisible = false;
                    break;
                }
            }
            if (allInvisible) {
                aggItem.setVisible(false);
            }
        }       
    }

    public void deselectAll() {
        cyNetwork.unselectAllNodes();
        cyNetwork.unselectAllEdges();
    }

    public void selectAll() {
        cyNetwork.selectAllNodes();
        cyNetwork.selectAllEdges();
    }

    protected void registerDefaultCommands() {
        // Override prefuse version. Do nothing.
    }

    public String getIdentifier() {
        return cyNetwork.getIdentifier();
    }


    public void postPaint(Display arg0, Graphics2D arg1) {

    }

    public void prePaint(Display arg0, Graphics2D arg1) {
        int fontSize = (int) (defaultFontSize * (1 / getScale()));
        if (fontSize != lastFontSize) {
            lastFontSize = fontSize;
            Iterator dividerLabelIt = vis.getVisualGroup(LINE_DIVIDER_LABELS)
                    .tuples();
            while (dividerLabelIt.hasNext()) {
                VisualItem label = (VisualItem) dividerLabelIt.next();
                label.setFont(FontLib.getFont("Verdana", fontSize));
                label.setValidated(false);
            }
        }
    }

    public static Rectangle2D getEndBounds(Visualization vis, String group) {
        Rectangle2D r = new Rectangle2D.Double();
        Rectangle2D r2 = new Rectangle2D.Double();
        Iterator iter = vis.visibleItems(group);
        if (iter.hasNext()) {
            VisualItem item = (VisualItem) iter.next();
            r.setRect(item.getEndX(), item.getEndY(), 1, 1);
        }
        while (iter.hasNext()) {
            VisualItem item = (VisualItem) iter.next();
            r2.setRect(item.getEndX(), item.getEndY(), 1, 1);
            Rectangle2D.union(r2, r, r);
        }
        return r;
    }

}