package cerebral.colors;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JPanel;

public class ColorScaleBar extends JPanel {

    private static final long serialVersionUID = 1L;

    ColorRange head;

    final static String DELIM = ",";

    public ColorScaleBar(ColorRange head) {
        this.head = head;
        setLayout(new GridBagLayout());
        relayout();
        setPreferredSize(new Dimension(600, 20));
    }
    
    public ColorRange getHead(){
        return head;
    }

    public void relayout() {
        removeAll();
        GridBagConstraints c = new GridBagConstraints();
        c.gridy = 0;
        c.fill = GridBagConstraints.BOTH;
        c.weighty = 1.0;

        double totalRange = 0;
        ColorRange range = head;
        if (range != null) {
            while (range.next() != null) {
                totalRange += range.getRange();
                range = range.next();
            }
            range = head;
            int i = 0;
            while (range.next() != null) {
                ColorRangeComponent crc = (ColorRangeComponent) range;
                c.gridx = i++;
                c.weightx = crc.getRange() / totalRange;
                add(crc, c);
                range = range.next();
            }
        }
        validate();
    }

    public static String saveString (ColorRange r){
        String s = new String();
        do {
            if (r instanceof ConstantRange) {
                ConstantRange cr = (ConstantRange) r;
                s += "c" + DELIM;
                s += Double.toString(cr.getMin()) + DELIM;
                s += Integer.toString(cr.color) + DELIM;
            } else if (r instanceof LinearRange) {
                LinearRange lr = (LinearRange) r;
                s += "l" + DELIM;
                s += Double.toString(lr.getMin()) + DELIM;
                s += Integer.toString(lr.colorMin) + DELIM;
                s += Integer.toString(lr.colorMax) + DELIM;
            } else if (r instanceof SigmoidalRange) {
                SigmoidalRange sr = (SigmoidalRange) r;
                s += "s" + DELIM;
                s += Double.toString(sr.getMin()) + DELIM;
                s += Integer.toString(sr.color) + DELIM;
                s += Double.toString(sr.s) + DELIM;
                s += Boolean.toString(sr.left) + DELIM;
            } else if (r instanceof TerminateRange) {
                s += "t" + DELIM;
                s += Double.toString(r.getMin()) + DELIM;
            }
            r = r.next();
        } while (r != null);
        return s;
    }
    public String saveString() {
        return saveString(head);     
    }

    public static ColorRange loadFromString(String saveString) {
        String[] s = saveString.split(DELIM);
        int i = 0;
        ColorRange prev = null;
        while (i < s.length){
            if (s[i].compareTo("c")==0){
                ConstantRange cr = new ConstantRange(Double.parseDouble(s[i+1]),
                        prev, Integer.parseInt(s[i+2]));
                prev = cr;
                i+=3;   
            }else if (s[i].compareTo("l")==0){
                LinearRange lr = new LinearRange(Double.parseDouble(s[i+1]),
                        prev, Integer.parseInt(s[i+2]), Integer.parseInt(s[i+3]));
                prev = lr;
                i+=4;
            }else if (s[i].compareTo("s")==0){
                SigmoidalRange sr = new SigmoidalRange(Double.parseDouble(s[i+1]),
                        prev, Integer.parseInt(s[i+2]), Double.parseDouble(s[i+3]), Boolean.parseBoolean(s[i+4]));
                prev =sr;
                i+=5;
            }else if (s[i].compareTo("t")==0){
                TerminateRange tr = new TerminateRange(Double.parseDouble(s[i+1]),
                        prev);
                prev = tr;
                i+=2;
            }
        }
        ColorRange head = prev;
        while (head.prev() != null){
            head = head.prev();
        }
        return head;

    }

}
