package cerebral.colors;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import prefuse.util.ColorLib;

public class ColorScaleDialog extends JFrame {
    
    private static final long serialVersionUID = 1L;
    private ColorScaleDialog outerThis;
    private ScaleEditor scaleEditor;
    public ColorScaleDialog (ColorRange head, String savedGallery){
        outerThis = this;
        setVisible(false);
        setDefaultLookAndFeelDecorated(true);
        JPanel panel = new JPanel();
        panel.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.gridx =0;
        c.gridy =0;
        c.gridwidth = 2;
        c.fill = GridBagConstraints.BOTH;
        c.weightx =1.0;
        c.weighty =1.0;
        
        scaleEditor = new ScaleEditor((ColorRangeComponent) head, savedGallery);
        panel.add(scaleEditor,c);
        c.gridwidth = 1;
        c.gridy=1;
        c.gridx =0;
        c.anchor = GridBagConstraints.LINE_END;
        c.fill = GridBagConstraints.NONE;
        c.weightx = 0;
        c.weighty = 0;
        c.gridx = 1;
        JPanel buttonPanel = new JPanel();
        
        
        JButton cancel = new JButton("Cancel");
        panel.add(cancel, c);
        cancel.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {
                outerThis.setVisible(false);
                outerThis.firePropertyChange("newScale", null, null);
                }            
        });
        buttonPanel.add(cancel);
        
        JButton apply = new JButton("Apply");
        panel.add(apply, c);
        apply.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e) {                
                outerThis.setVisible(false);
                if (scaleEditor.userEdited()){
                    scaleEditor.gallery.addUserScale(scaleEditor.csLegend.csb.saveString());
                }
                outerThis.firePropertyChange("newScale", null, scaleEditor.csLegend.csb.saveString());
               }
        });
        buttonPanel.add(apply);
        
        panel.add(buttonPanel,c);
        setContentPane(panel);
    }
    
    public String getSaveGallery(){
        return scaleEditor.gallery.getUserScales();
    }
    public void showDialog(){
        setSize(new Dimension(600,600));
        setVisible(true);
    }
    
	public static void main(String[] args) {
		// Schedule a job for the event-dispatching thread:
		// creating and showing this application's GUI.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				// Make sure we have nice window decorations.
				JFrame.setDefaultLookAndFeelDecorated(true);

			
				ColorRange head = new ConstantRange(-5.0,  null, ColorLib.rgb(255,0,0));
				ColorRange next = new ConstantRange(-3.0,  head, ColorLib.rgb(0,255,0));
				ColorRange n1 = new ConstantRange(-2.0,  next, ColorLib.rgb(0,0,255));
				ColorRange n2 = new ConstantRange(1.0,  n1, ColorLib.rgb(255,0,255));
				ColorRange terminator = new TerminateRange(7.0, n2);				
				ColorScaleDialog csb = new ColorScaleDialog(head, "");
				csb.showDialog();
			}
		});
	}
    
    

}
