package cerebral.colors;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.font.TextLayout;
import java.text.NumberFormat;

import javax.swing.JComponent;

public class ColorScaleLabels extends JComponent {
	
    private static final long serialVersionUID = 1L;
    private ColorRange head;
	private final int lineLength = 3;

	ColorScaleLabels(ColorRange head) {
		this.head = head;
		NumberFormat.getInstance().setMaximumFractionDigits(2);
		setPreferredSize(new Dimension(500,20));
		setOpaque(false);
	}

	public void paint(Graphics g1) {
		Graphics2D g = (Graphics2D) g1;
		g.setColor(getParent().getBackground());
		g.fillRect(0,0, getWidth(), getHeight());
		g.setColor(Color.black);
		// First priority is to paint the beginning and end labels
		double minVal = head.getMin(); ;
		String min = NumberFormat.getInstance().format(minVal);

		// draw the down line
		g.drawLine(2, 0, 2, lineLength);

		// see how big a font we can use
		int textHeight = getHeight() - (lineLength+2);
		int fontSize = 16;
		do {
			TextLayout tl = new TextLayout(min, new Font("Verdana", Font.PLAIN,
					fontSize), g.getFontRenderContext());
			if (tl.getBounds().getHeight() < textHeight) {
				break;
			}
			fontSize--;
		} while (fontSize > 6);

		Font font = new Font("Verdana", Font.PLAIN, fontSize);
		TextLayout tl = new TextLayout(min, font, g.getFontRenderContext());
		tl.draw(g, 0, (float) (tl.getBounds().getHeight() + lineLength + 1));

		double leftEdge = tl.getBounds().getWidth();

		// draw the endLabel;
		g.drawLine(getWidth() - 2, 0, getWidth() - 2, lineLength);

		ColorRange curRange = head;
		while (curRange.next() != null){
			curRange = curRange.next();
		}
		String max = NumberFormat.getInstance().format(curRange.getMin());
		tl = new TextLayout(max, font, g.getFontRenderContext());
		tl.draw(g, (float) (getWidth() - 1 - tl.getBounds().getWidth() - 2),
				(float) (tl.getBounds().getHeight() + lineLength + 1));

		double rightEdge = getWidth() - 1 - tl.getBounds().getWidth();

		// Find the total numeric range

		double totalRange = 0;
		curRange = head;
		while (curRange.next() != null){
				totalRange += curRange.getRange();
				curRange = curRange.next();
		}
	
		double range = head.getRange();

		// now go through and draw as many nonoverlapping labels as possible
		curRange = head.next();
		while (curRange.next() != null) {
			String label = NumberFormat.getInstance().format(curRange.getMin());
			tl = new TextLayout(label, font, g.getFontRenderContext());

			int downLinePos = (int) (range / totalRange * getWidth());
			g.drawLine(downLinePos, 0, downLinePos, lineLength);
			range += curRange.getRange();
			
			if (downLinePos - tl.getBounds().getWidth() / 2 > (leftEdge + 2)
					&& downLinePos + tl.getBounds().getWidth() / 2 < (rightEdge-2)) {
				leftEdge = (downLinePos + tl.getBounds().getWidth() / 2);
				tl.draw(g, (float) (downLinePos - tl.getBounds().getWidth() / 2), (float) (tl.getBounds()
						.getHeight()
						+ lineLength + 1));
			}
			curRange = curRange.next();
		}
		super.paint(g);
	}

}
