package cerebral.controls;

import java.awt.event.MouseEvent;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import cerebral.Cerebral;

import prefuse.Visualization;
import prefuse.data.tuple.TupleSet;
import prefuse.util.ui.UILib;
import prefuse.visual.AggregateItem;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;

import prefuse.controls.ControlAdapter;

public class ClickToSelect extends ControlAdapter {
    private Cerebral cytoPrefuse;

    private Set setItems;

    public ClickToSelect(Cerebral cytoPrefuse) {
        this.cytoPrefuse = cytoPrefuse;
        setItems = new HashSet();
    }

    public void mouseClicked(MouseEvent e) {
        if (UILib.isButtonPressed(e, LEFT_MOUSE_BUTTON)
                && e.getClickCount() == 1) {
            if (!e.isShiftDown()) {
                cytoPrefuse.selectAll();
            }
            //ugh.  No way to detect VizMapper changes after Cytoscape2.5.0
            //Hack here to force an update when the user clicks the background of the visualization
            cytoPrefuse.getVisualization().run("applyVizMapper");
        }
    }

    public void itemDragged(VisualItem item, MouseEvent e) {

    }

    public void itemClicked(VisualItem item, MouseEvent e) {
        if (UILib.isButtonPressed(e, LEFT_MOUSE_BUTTON)
                && e.getClickCount() == 1) {
            if (item instanceof NodeItem || item instanceof EdgeItem) {
                setItems.clear();
                setItems.add(item);
                Visualization vis = item.getVisualization();
                TupleSet ts = vis.getFocusGroup(Cerebral.SELECTED);
                boolean shift = e.isShiftDown();
                if (!shift) {
                    cytoPrefuse.setSelectedItems(setItems);
                } else {
                    cytoPrefuse.selectInCytoscape(setItems.toArray(), !ts
                            .containsTuple(item));
                }
            } else {
                if (!e.isShiftDown()) {
                    if (!(item instanceof AggregateItem)){
                        cytoPrefuse.selectAll();
                    }
                }
            }
        } else if (UILib.isButtonPressed(e, LEFT_MOUSE_BUTTON)
                && e.getClickCount() == 2) {
            if (item instanceof AggregateItem) {
                Visualization vis = item.getVisualization();
                TupleSet ts = vis.getFocusGroup(Cerebral.SELECTED);
                boolean shift = e.isShiftDown();

                AggregateItem aggItem = (AggregateItem) item;

                Iterator itemsIt = aggItem.items();
                setItems.clear();
                boolean allSelected = true;

                while (itemsIt.hasNext()) {
                    VisualItem groupMember = (VisualItem) itemsIt.next();
                    if (groupMember instanceof NodeItem) {
                        setItems.add(groupMember);
                        if (!ts.containsTuple(groupMember)) {
                            allSelected = false;
                        }
                    }
                }
                if (!shift) {
                    cytoPrefuse.setSelectedItems(setItems);                   
                } else {
                    cytoPrefuse.selectInCytoscape(setItems.toArray(),
                            !allSelected);
                }
            }
        }
    }
}
