package cerebral.controls;

import java.awt.Cursor;
import java.awt.event.MouseEvent;

import cerebral.ViewCoordinator;

import prefuse.Display;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;


import prefuse.controls.ControlAdapter;


public class EdgeHighlightControl extends ControlAdapter {
	private String activity;
	private String group;
	private ViewCoordinator viewCoordinator;
	
	public EdgeHighlightControl(String activity, String group,  ViewCoordinator viewCoordinator) {
	        this.activity = activity;
	        this.group = group;
	        this.viewCoordinator = viewCoordinator;
	    }

	/**
     * @see prefuse.controls.Control#itemEntered(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemEntered(VisualItem item, MouseEvent e) {
        if ( item instanceof EdgeItem ){
            setNeighborHighlight((EdgeItem)item, true);
            Display d = (Display) e.getSource();
            d.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        }
    }
    
    /**
     * @see prefuse.controls.Control#itemExited(prefuse.visual.VisualItem, java.awt.event.MouseEvent)
     */
    public void itemExited(VisualItem item, MouseEvent e) {
        if ( item instanceof EdgeItem ){
            setNeighborHighlight((EdgeItem)item, false);
            Display d = (Display)e.getSource();
            d.setCursor(Cursor.getDefaultCursor());
        }
    }
    
    /**
     * Set the highlighted state of the neighbors of a node.
     * @param n the node under consideration
     * @param state the highlighting state to apply to neighbors
     */
    protected void setNeighborHighlight(EdgeItem e, boolean state) {
        viewCoordinator.select(group, e, state);

       NodeItem source =  e.getSourceItem();
       viewCoordinator.select(group, source,state);

       NodeItem dest =  e.getTargetItem();
       viewCoordinator.select(group, dest,state);

       
       if (activity != null){
           viewCoordinator.run(activity);
           viewCoordinator.run("repaint");
       }
       
       
     }
}
