package cerebral.parallelCoordinates.render;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.text.NumberFormat;

import prefuse.render.Renderer;
import prefuse.util.ColorLib;
import prefuse.util.StrokeLib;
import prefuse.visual.VisualItem;
import cerebral.colors.ColorManager;
import cerebral.parallelCoordinates.PCLayout;
import cerebral.parallelCoordinates.YDisplay;

public class ScaleLineRenderer implements Renderer {

	private Line2D line = new Line2D.Double();
	private YDisplay yDisplay;
	private NumberFormat nf = NumberFormat.getInstance();
	private Rectangle2D clip = new Rectangle2D.Double();
	private String commonPrefix;

	public ScaleLineRenderer(YDisplay yDisplay, String commonPrefix) {
		this.yDisplay = yDisplay;
		nf.setMaximumFractionDigits(2);
		clip.setRect(-1000000, 10, 2000000, 100);
		this.commonPrefix = commonPrefix;
	}

	public void render(Graphics2D g, VisualItem item) {
		g.setClip(clip);

		// BasicStroke stroke = item.getStroke();
		float[] dashPat = { 2f, 6f };
		BasicStroke stroke = StrokeLib.getStroke(2, dashPat);
		Color color = ColorLib.getColor(ColorManager.increasedContrast(ColorLib
				.color(item.getVisualization().getDisplay(0).getBackground())));

		Color bg = item.getVisualization().getDisplay(0).getBackground();

		drawAxis(g, stroke, color, bg, -2);
		drawAxis(g, stroke, color, bg, 2);
		g.setClip(null);

		RenderUtil.drawTextAtTop(g, commonPrefix, bg, Color.black, 0,
				(float) clip.getMaxY() + PCLayout.MARGIN);

	}

	private void drawAxis(Graphics2D g, BasicStroke stroke, Color color,
			Color bg, double val) {
		double pos = yDisplay.getPosition(val);
		g.setStroke(stroke);
		g.setColor(color);

		line.setLine(30, pos, 10000, pos);
		g.draw(line);
		RenderUtil.drawText(g, nf.format(val), bg, 0, (float) pos);
	}

	public boolean locatePoint(Point2D p, VisualItem item) {
		return false;
	}

	public void setBounds(VisualItem item) {
		item.setBounds(0, -100000, 10000, 200000);
	}

	public void setBounds(int top, double height) {
		clip.setRect(-100000, top, 200000, height);
	}

}
